fn coerce_mutbls<'db>(
from_mutbl: Mutability,
to_mutbl: Mutability,
) -> RelateResult<'db, ()>Expand description
Coercing a mutable reference to an immutable works, while
coercing &T to &mut T should be forbidden.
fn coerce_mutbls<'db>(
from_mutbl: Mutability,
to_mutbl: Mutability,
) -> RelateResult<'db, ()>Coercing a mutable reference to an immutable works, while
coercing &T to &mut T should be forbidden.