struct CoerceVisitor<'a, D> {
delegate: &'a D,
errored: bool,
unsize_did: TraitId,
coerce_unsized_did: TraitId,
span: Span,
}Expand description
Recursively visit goals to decide whether an unsizing is possible.
Breaks when it isn’t, and an error should be raised.
Continues when an unsizing ok based on an implementation of the Unsize trait / lang item.
Fields
delegate: &'a Derrored: boolWhether the coercion is impossible. If so we sometimes still try to coerce in these cases to emit better errors. This changes the behavior when hitting the recursion limit.
unsize_did: TraitIdcoerce_unsized_did: TraitIdspan: SpanTrait Implementations
Sourceimpl<'a, 'db, D: CoerceDelegate<'db>> ProofTreeVisitor<'db> for CoerceVisitor<'a, D>
impl<'a, 'db, D: CoerceDelegate<'db>> ProofTreeVisitor<'db> for CoerceVisitor<'a, D>
type Result = ControlFlow<()>
fn span(&self) -> Span
fn visit_goal(&mut self, goal: &InspectGoal<'_, 'db>) -> Self::Result
fn on_recursion_limit(&mut self) -> Self::Result
fn config(&self) -> InspectConfig